from Crypto.Cipher import AES
import math
import io
import auth
import constants
import md5
import os

def validPerms(username, perms, t):
    perms = perms.split(',')
    m = {}
    for perm in perms:
        key,val = perm.split(':')
        m[key] = val
    if m.has_key(username):
        return t in m[username]
    return False

def validReadPerms(username, perms):
    return validPerms(username, perms, 'r')

def validWritePerms(username, perms):
    return validPerms(username, perms, 'w')

def validReadWritePerms(username, perms):
    return validPerms(username, perms, 'rw')

def checksum(filename):
    if (not auth.master.has_key(filename)):
        return True
    csum = ''
    key,check_sum = get(filename)
    try:
        data = read(filename, key)
        csum, perms, contents = parse(data)
        check_sum = getNewChecksumContents(contents)
    except IOError, e:
        csum = getNewChecksum(filename, key)
    if (check_sum == csum):
        return True
    return False 

def listdir(filename):
    if os.path.isdir(filename):
        dir_content = os.listdir(filename)
        return '\n'.join([a for a in [io.getDecName(p) for p in dir_content] if '.master' not in a])
    else:
        print "File directory does not exist!"
        return ''

def updateDir(filename):
    csum = getNewChecksum(filename,'')
    io.setCsum(filename,csum)

def getNewChecksum(filename, key):
    try:
        data = read(filename, key)
        csum, perms, contents = parse(data)
        return csum
    except IOError, e:
        #this is a directory
        contents = listdir(filename)
        m = md5.new(contents)
        csum = m.digest()
        return csum.encode('hex')

def getNewChecksumContents(contents):
    m = md5.new(contents)
    csum = m.digest()
    return csum.encode('hex')

def getEncName(filename, key):
    if not key:
        return '../tmp'
    path = filename.split('/')
    last = path[-1]
    til = path[:len(path)-1]
    cur = []
    result = ''
    for part in til:
        cur += [part]
        result += '/' + io.encrypt(io.pad(part),io.get('/'.join(cur))[0]).encode('hex')
    en = '/' + io.encrypt(io.pad(last),key).encode('hex')
    return '../tmp' + result + en

def getDecName(filename):
    path = filename.split('/')
    while '..' in path or 'tmp' in path:
        del path[0]
    cur = ['../tmp']
    result = ''
    for part in path:
        cur += [part]
        try:
            result += '/' + io.decrypt(part.decode('hex'),io.get('/'.join(cur))[0]).rstrip('\0')
        except TypeError, e:
            result += '/' + part
    return result

# def getEncName(filename,key):
#     path = filename.split('/')
#     epath = ['../tmp'] + [io.encrypt(io.pad(part),key).encode('hex') for part in path if part]
#     return '/'.join(epath)

# def getDecName(encoded,key):
#     path = encoded.split('/')
#     epath = [io.decrypt(part.decode('hex'),key).rstrip('\0') for part in path if part]
#     return '/'.join(epath)

def read(filename, key):
    with open(filename, 'r') as fd:
        return decrypt(fd.read()[len(constants.HEADER):], key).rstrip('\0')

def parse(data):
    csum,perms,content = data.split(constants.SEPARATOR)
    return (csum,perms,content)

def format(csum, perms, content):
    csum = pad(csum) + constants.SEPARATOR
    perms = pad(perms) + constants.SEPARATOR
    content = pad(content)
    return csum + perms + content

def write(filename, content, key):
    cipher = constants.HEADER + encrypt(pad(content), key)
    with open(filename, 'w') as fh:
        fh.seek(0)
        fh.write(cipher)
        fh.truncate()
    auth.master[filename] = (key, '1')

def encrypt(data, key):
    obj = AES.new(key, AES.MODE_CBC, auth.IV)
    return obj.encrypt(data)

def decrypt(data, key):
    obj = AES.new(key, AES.MODE_CBC, auth.IV)
    return obj.decrypt(data)

def pad(data):
    n = int(math.ceil(len(data)/16.0))
    return data + (16*n-len(data))*'\0'

def setKey(filename, key, csum):
    auth.master[filename] = (key,csum)

def setCsum(filename, csum):
    key,old = get(filename)
    setKey(filename,key,csum)

def get(filename):
    if auth.master.has_key(filename):
        return auth.master[filename]
    return ('0'*16,0)

def rm(filename, efname):
    del auth.master[filename]
    del auth.master[efname]

def mv(old, old_e, new, new_e):
    auth.master[new] = auth.master[old]
    auth.master[new_e] = auth.master[old_e]

def has(filename):
    return auth.master.has_key(filename)

def writeKeys():
    auth.writeKeys(auth.key)

def getKeys():
    return auth.master

def genKey():
    return auth.genKey()
