#!/usr/bin/python

import errno
import tempfile
import os.path
import shlex
import subprocess
import sys
import urllib2
import auth
import io
import share
import constants

red = "\033[0;31m"
green = "\033[0;32m"
yellow = "\033[0;33m"
blue = "\033[0;34m"
normal = "\033[0m"

prompt = "%spyshell %s$%s " % (blue, green, normal, )
count = 0

class CommandError(Exception):
    pass

def shareKey(args):
    print share.shareKey(args[0], args[1], args[2])

def unlock(args):
    share.unlock(args[0],args[1])

def ls(args):
    root = ''
    if len(args) != 0:
        root = args[0]
    filename = ''
    if root:
        filename = io.getEncName(os.path.expanduser(root),io.get(root)[0])
    else:
        filename = io.getEncName(0,0)
    if os.path.isdir(filename):
        if not io.checksum(filename) and root != '':
            print "Unauthorized modification to this directory!"
        dir_content = os.listdir(filename)
        if not root:
            print '\n'.join([a for a in [io.getDecName(p) for p in dir_content] if '.master' not in a])
        else:
            print '\n'.join([io.getDecName(filename + '/' + p) for p in dir_content])
    else:
        print root

def mkdir(args):
    rkey = io.genKey()
    unefn = os.path.expanduser(args[0])
    #TODO: COMPUTE CHECKSUM
    filename = io.getEncName(unefn, rkey)
    subprocess.call('mkdir %s'%filename,shell=True)
    csum = io.getNewChecksum(filename, rkey)
    io.setKey(filename, rkey, csum)
    io.setKey(unefn, rkey, csum)
    io.writeKeys()
 

def rm(args):
    if len(args) != 1:
        raise CommandError("Usage: rm <file>")
    unefn = os.path.expanduser(args[0])
    rkey = io.get(unefn)[0]
    filename = io.getEncName(unefn, rkey)
    subprocess.call('rm -r %s'%filename,shell=True)
    io.rm(unefn, filename)
    io.updateDir('/'.join(filename.split('/')[:-1]))
    io.writeKeys()

def mv(args):
    if len(args) != 2:
        raise CommandError("Usage: mv <old_file> <new_file>")
    old_unefn = os.path.expanduser(args[0])
    rkey = io.get(old_unefn)[0]
    old_filename = io.getEncName(old_unefn, rkey)
    new_unefn = os.path.expanduser(args[1])
    new_filename = io.getEncName(new_unefn, rkey)
    subprocess.call('mv %s %s'%(old_filename,new_filename),shell=True)
    io.mv(old_unefn, old_filename, new_unefn, new_filename)
    io.updateDir('/'.join(new_filename.split('/')[:-1]))
    io.writeKeys()

def cat(args):
    if len(args) != 1:
        raise CommandError("Usage: cat <file>")
    unefn = os.path.expanduser(args[0])
    rkey = io.get(unefn)[0]
    filename = io.getEncName(unefn, rkey)
    #TODO: CHECKSUM
    if (not io.checksum(filename)):
        print "Checksum failed! Unauthorized modification!"
        return
    try:
        with open(filename,'r') as fr:
            content = fr.read()
            if len(content) >= 16 and content[:16] == constants.HEADER:
                data = io.read(filename, rkey)
                csum, perms, contents = io.parse(data)
                if (not io.validReadPerms(auth.username, perms)):
                    print "Permission denied. No read permissions!"
                else:
                    print contents
            else:
                print content
    except IOError, e:
        print "Failed to open %s" % (filename, )

def vi(args):
    perms = ''
    if len(args) != 1:
        raise CommandError("Usage: vi <file>")
    rkey = io.genKey()
    unefn = os.path.expanduser(args[0])
    if io.has(unefn):
        rkey = io.get(unefn)[0]
    filename = io.getEncName(unefn,rkey)
    #TODO: CHECKSUM
    if (not io.checksum(filename)):
        print "Checksum failed! Unauthorized modification!"
        return
    try:
        t, temp = tempfile.mkstemp()
        content = ""
        if os.path.isfile(filename):
            with open(filename,'r+') as fr:
                content = fr.read()
                if len(content) >= 16 and content[:16] == constants.HEADER:
                    fr.seek(0)
                    data = io.read(filename, rkey)
                    csum, perms, contents = io.parse(data)
                    fr.write(contents)
                    fr.truncate()
            if (io.validReadWritePerms(auth.username, perms)):
                subprocess.call('vi %s'%filename,shell=True)
            elif (io.validReadPerms(auth.username, perms) and not io.validWritePerms(auth.username, perms)):
                print "Read only no Write permissions"
                subprocess.call('vi -R %s'%filename,shell=True)
            elif (io.validWritePerms(auth.username, perms) and not io.validReadPerms(auth.username, perms)):
                with open(filename,'r+') as fr:
                    fr.seek(0)
                    fr.write('')
                    fr.truncate()
                    print "Write only no Read permissions"
            else:
                print "How did you get the key?"
                return
            temp = filename
        elif os.path.isfile(unefn):
            subprocess.call('vi %s'%unefn,shell=True)
            os.rename(unefn, filename)
            temp = filename
        else:
            perms = auth.username + ':rw'
            subprocess.call('vi %s'%temp,shell=True)

        csum = ''
        with open(os.path.expanduser(temp),'r') as fr:
            content = fr.read()
            csum = io.getNewChecksumContents(content)
            fcontent = io.format(csum, perms, content) #TODO: perm = '', placeholder
            io.write(filename, fcontent, rkey)
        #TODO: RECOMPUTE CHECKSUM
        io.setKey(unefn, rkey, csum)
        io.setKey(filename, rkey, csum)
        io.updateDir('/'.join(filename.split('/')[:-1]))
        io.writeKeys()
            
    except IOError, e:
        print "Failed to open %s" % (filename, )

def show_help(args):
    print "Available commands:"
    for cmd in commands:
        print "- " + cmd

commands = {
    'cat': cat,
    'vi': vi,
    'mkdir': mkdir,
    'ls': ls,
    'rm': rm,
    'mv': mv,
    'share': shareKey,
    'unlock': unlock,
    'login': auth.login,
    'logout': auth.logout,
    'help': show_help,
}

def run_command(cmd, args):
    if cmd in commands:
        try:
            commands[cmd](args)
        except CommandError, e:
            print "%s%s%s" % (red, e.message, red, )
    else:
        print "%s%s%s: %scommand not found%s" % (yellow, cmd, normal, red, normal, )

def shell():
    try:
        print green+"Please login by inputting username password"
        cmd_str = raw_input(prompt)
        args = shlex.split(cmd_str)
        run_command('login', args)
        while True:
            cmd_str = raw_input(prompt)
            args = shlex.split(cmd_str)
            if not args: continue
            cmd = args[0]
            run_command(cmd, args[1:])
    except EOFError:
        print "\nGoodbye!"

if __name__ == '__main__':
    if len(sys.argv) > 1:
        count = int(sys.argv[1])
        print "Starting shell (count %d)" % (count, )
    else:
        count = 1
    shell()

